# Hacemos una búsqueda en malla
# de los parámetros C y gamma para
# una MVS

import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.svm import SVC
from sklearn.model_selection import GridSearchCV
import matplotlib.pyplot as plt
import pandas as pd

# Leemos los datos de entrada
D = np.loadtxt("datosOk.txt")
X = D[:,0:2]
y = D[:,2]

# Lo separamos en datos de entrenamiento y prueba
# X_entrena, X_prueba, y_entrena, y_prueba = train_test_split( X, y, random_state=2 )

gammas = [0.1, 1, 5, 10, 15]
Cs = [0.1, 1, 10, 100, 1000 ]
parametros = {"C": Cs, "gamma":gammas}

clasificador = SVC( )

# Hacemos la búsqueda en la malla
r = GridSearchCV( clasificador, parametros )
r.fit( X, y )

print( r.cv_results_ ) 

def make_heatmap(ax, gs, is_sh=False, make_cbar=False):
	"""Helper to make a heatmap."""
	results = pd.DataFrame(gs.cv_results_)
	results[["param_C", "param_gamma"]] = results[["param_C", "param_gamma"]].astype(
		np.float64
	)
	if is_sh:
		# SH dataframe: get mean_test_score values for the highest iter
		scores_matrix = results.sort_values("iter").pivot_table(
			index="param_gamma",
			columns="param_C",
			values="mean_test_score",
			aggfunc="last",
		)
	else:
		scores_matrix = results.pivot(
			index="param_gamma", columns="param_C", values="mean_test_score"
		)

	im = ax.imshow(scores_matrix)

	ax.set_xticks(np.arange(len(Cs)))
	ax.set_xticklabels(["{:.1E}".format(x) for x in Cs])
	ax.set_xlabel("C", fontsize=15)

	ax.set_yticks(np.arange(len(gammas)))
	ax.set_yticklabels(["{:.1E}".format(x) for x in gammas])
	ax.set_ylabel("gamma", fontsize=15)

	# Rotate the tick labels and set their alignment.
	plt.setp(ax.get_xticklabels(), rotation=45, ha="right", rotation_mode="anchor")

	if is_sh:
		iterations = results.pivot_table(
			index="param_gamma", columns="param_C", values="iter", aggfunc="max"
		).values
		for i in range(len(gammas)):
			for j in range(len(Cs)):
				ax.text(
					j,
					i,
					iterations[i, j],
					ha="center",
					va="center",
					color="w",
					fontsize=20,
				)

	if make_cbar:
		fig.subplots_adjust(right=0.8)
		cbar_ax = fig.add_axes([0.85, 0.15, 0.05, 0.7])
		fig.colorbar(im, cax=cbar_ax)
		cbar_ax.set_ylabel("mean_test_score", rotation=-90, va="bottom", fontsize=15)


fig, axes = plt.subplots( )

make_heatmap(axes, r, make_cbar=True)

axes.set_title("Busqueda en malla")

plt.show()

